@@ -31,6 +31,19 @@
 #include "watchdog/wd_ipc_conn.h"
 #include "watchdog/wd_commands.h"
 
+/*
+ * These lock can only be acquired by
+ * coordinator watchdog node on standby
+ * watchdog node.
+ */
+typedef enum WD_LOCK_STANDBY_TYPE
+{
+	WD_INVALID_LOCK,
+	/* currently we have only one lock */
+	WD_FOLLOW_PRIMARY_LOCK
+}WD_LOCK_STANDBY_TYPE;
+
+
 extern WdCommandResult wd_start_recovery(void);
 extern WdCommandResult wd_end_recovery(void);
 extern WDFailoverCMDResults wd_send_failback_request(int node_id, unsigned char flags);
@@ -60,4 +73,8 @@ extern void set_watchdog_node_escalated(void);
 extern void reset_watchdog_node_escalated(void);
 extern bool get_watchdog_node_escalation_state(void);
 extern size_t wd_ipc_get_shared_mem_size(void);
+
+extern WdCommandResult wd_lock_standby(WD_LOCK_STANDBY_TYPE lock_type);
+extern WdCommandResult wd_unlock_standby(WD_LOCK_STANDBY_TYPE lock_type);
+
 #endif							/* WD_INTERNAL_COMMANDS_H */
