@@ -72,6 +72,8 @@ static WDFailoverCMDResults wd_get_failover_result_from_data(WDIPCCmdResult * re
 static WDFailoverCMDResults wd_issue_failover_command(char *func_name, int *node_id_set,
 													  		int count, unsigned char flags);
 
+static WdCommandResult wd_send_locking_command(WD_LOCK_STANDBY_TYPE lock_type,
+															bool acquire);
 
 void
 wd_ipc_initialize_data(void)
@@ -552,3 +554,37 @@ wd_internal_get_watchdog_local_node_state(void)
 {
 	return get_watchdog_local_node_state(pool_config->wd_authkey);
 }
+
+static WdCommandResult
+wd_send_locking_command(WD_LOCK_STANDBY_TYPE lock_type, bool acquire)
+{
+	WDExecCommandArg wdExecCommandArg[2];
+
+	strncpy(wdExecCommandArg[0].arg_name, "StandbyLockType", sizeof(wdExecCommandArg[0].arg_name) - 1);
+	snprintf(wdExecCommandArg[0].arg_value, sizeof(wdExecCommandArg[0].arg_name) - 1, "%d",lock_type);
+
+	strncpy(wdExecCommandArg[1].arg_name, "LockingOperation", sizeof(wdExecCommandArg[1].arg_name) - 1);
+	snprintf(wdExecCommandArg[1].arg_value, sizeof(wdExecCommandArg[1].arg_name) - 1,
+			 "%s",acquire?"acquire":"release");
+
+	ereport(DEBUG1,
+			(errmsg("sending standby locking request to watchdog")));
+
+	return wd_execute_cluster_command(WD_COMMAND_LOCK_ON_STANDBY, 2, wdExecCommandArg);
+}
+
+WdCommandResult
+wd_lock_standby(WD_LOCK_STANDBY_TYPE lock_type)
+{
+	if (pool_config->use_watchdog)
+		return wd_send_locking_command(lock_type, true);
+	return COMMAND_OK;
+}
+
+WdCommandResult
+wd_unlock_standby(WD_LOCK_STANDBY_TYPE lock_type)
+{
+	if (pool_config->use_watchdog)
+		return wd_send_locking_command(lock_type, false);
+	return COMMAND_OK;
+}
